@@ -1,16 +1,22 @@
-from typing import List, Dict, Optional
+from typing import List, Dict, Optional, Tuple
 import time
 from dataclasses import dataclass
 import backoff  # type: ignore
-import requests
-from requests.exceptions import Timeout, HTTPError, ConnectionError
+import httpx
+from httpx import HTTPStatusError, ConnectError, TimeoutException
 import spectacles.utils as utils
 from spectacles.types import JsonDict
 from spectacles.logger import GLOBAL_LOGGER as logger
 from spectacles.exceptions import SpectaclesException, LookerApiError
 
+DEFAULT_API_VERSION = 4.0
 TIMEOUT_SEC = 300
-BACKOFF_EXCEPTIONS = (Timeout, HTTPError, ConnectionError)
+MAX_ASYNC_CONNECTIONS = 200
+BACKOFF_EXCEPTIONS = (
+    TimeoutException,
+    HTTPStatusError,
+    ConnectError,
+)
 
 
 @dataclass(frozen=True)  # Token is immutable
@@ -18,6 +24,7 @@ class AccessToken:
     access_token: str
     token_type: str
     expires_in: int
+    refresh_token: Optional[str]
     expires_at: float
 
     def __str__(self) -> str:
@@ -46,13 +53,15 @@ class LookerClient:
 
     def __init__(
         self,
+        async_client: httpx.AsyncClient,
         base_url: str,
         client_id: str,
         client_secret: str,
         port: Optional[int] = None,
-        api_version: float = 3.1,
+        api_version: float = DEFAULT_API_VERSION,
     ):
-        supported_api_versions = [3.1]
+        self.async_client = async_client
+        supported_api_versions = [4.0]
         if api_version not in supported_api_versions:
             raise SpectaclesException(
                 name="unsupported-api-version",
@@ -80,7 +89,6 @@ class LookerClient:
         self.client_secret: str = client_secret
         self.api_version: float = api_version
         self.access_token: Optional[AccessToken] = None
-        self.session: requests.Session = requests.Session()
         self.workspace: str = "production"
 
         self.authenticate()
@@ -98,12 +106,11 @@ class LookerClient:
 
         url = utils.compose_url(self.api_url, path=["login"])
         body = {"client_id": self.client_id, "client_secret": self.client_secret}
-        self.session.auth = NullAuth()
         # This should not use `self.post` or it will create a recursive loop
-        response = self.session.post(url=url, data=body, timeout=TIMEOUT_SEC)
+        response = httpx.post(url=url, data=body, timeout=TIMEOUT_SEC)
         try:
             response.raise_for_status()
-        except requests.exceptions.HTTPError:
+        except httpx.HTTPStatusError as error:
             raise LookerApiError(
                 name="unable-to-authenticate",
                 title="Couldn't authenticate to the Looker API.",
@@ -113,16 +120,16 @@ class LookerClient:
                     "Check that your credentials are correct and try again."
                 ),
                 response=response,
-            )
+            ) from error
 
         result = response.json()
         if "expires_at" not in result:
             # Calculate the expiration time with a one-minute buffer
             result["expires_at"] = time.time() + result["expires_in"] - 60
         self.access_token = AccessToken(**result)
-        self.session.headers = {  # type: ignore
-            "Authorization": f"token {self.access_token}"
-        }
+        self.async_client.headers = httpx.Headers(
+            {"Authorization": f"token {self.access_token}"}
+        )
 
         looker_version = self.get_looker_release_version()
         logger.info(
@@ -130,33 +137,29 @@ class LookerClient:
             f"using Looker API {self.api_version}"
         )
 
-    @backoff.on_exception(
-        backoff.expo,
-        BACKOFF_EXCEPTIONS,
-        max_tries=3,
-    )
-    def request(self, method: str, url: str, *args, **kwargs) -> requests.Response:
+    @backoff.on_exception(backoff.expo, BACKOFF_EXCEPTIONS, max_tries=3)
+    async def request(self, method: str, url: str, *args, **kwargs) -> httpx.Response:
         if self.access_token and self.access_token.expired:
             logger.debug("Looker API access token has expired, requesting a new one")
             self.authenticate()
             if self.workspace == "dev":
-                self.update_workspace("dev")
-        return self.session.request(method, url, *args, **kwargs)
+                await self.update_workspace("dev")
+        return await self.async_client.request(method, url, *args, **kwargs)
 
-    def get(self, url, *args, **kwargs) -> requests.Response:
-        return self.request("GET", url, *args, **kwargs)
+    async def get(self, url, *args, **kwargs) -> httpx.Response:
+        return await self.request("GET", url, *args, **kwargs)
 
-    def post(self, url, *args, **kwargs) -> requests.Response:
-        return self.request("POST", url, *args, **kwargs)
+    async def post(self, url, *args, **kwargs) -> httpx.Response:
+        return await self.request("POST", url, *args, **kwargs)
 
-    def patch(self, url, *args, **kwargs) -> requests.Response:
-        return self.request("PATCH", url, *args, **kwargs)
+    async def patch(self, url, *args, **kwargs) -> httpx.Response:
+        return await self.request("PATCH", url, *args, **kwargs)
 
-    def put(self, url, *args, **kwargs) -> requests.Response:
-        return self.request("PUT", url, *args, **kwargs)
+    async def put(self, url, *args, **kwargs) -> httpx.Response:
+        return await self.request("PUT", url, *args, **kwargs)
 
-    def delete(self, url, *args, **kwargs) -> requests.Response:
-        return self.request("DELETE", url, *args, **kwargs)
+    async def delete(self, url, *args, **kwargs) -> httpx.Response:
+        return await self.request("DELETE", url, *args, **kwargs)
 
     def get_looker_release_version(self) -> str:
         """Gets the version number of connected Looker instance.
@@ -169,10 +172,10 @@ class LookerClient:
 
         url = utils.compose_url(self.api_url, path=["versions"])
 
-        response = self.get(url=url, timeout=TIMEOUT_SEC)
+        response = httpx.get(url=url, timeout=TIMEOUT_SEC)
         try:
             response.raise_for_status()
-        except requests.exceptions.HTTPError:
+        except httpx.HTTPStatusError as error:
             raise LookerApiError(
                 name="unable-to-get-version",
                 title="Couldn't get Looker's release version.",
@@ -182,11 +185,11 @@ class LookerClient:
                     "Please try again."
                 ),
                 response=response,
-            )
+            ) from error
 
         return response.json()["looker_release_version"]
 
-    def get_workspace(self) -> str:
+    async def get_workspace(self) -> str:
         """Gets the session workspace.
 
         Args:
@@ -197,10 +200,10 @@ class LookerClient:
         """
         logger.debug("Getting the workspace in use by this session")
         url = utils.compose_url(self.api_url, path=["session"])
-        response = self.get(url=url, timeout=TIMEOUT_SEC)
+        response = await self.get(url=url, timeout=TIMEOUT_SEC)
         try:
             response.raise_for_status()
-        except requests.exceptions.HTTPError:
+        except httpx.HTTPStatusError as error:
             raise LookerApiError(
                 name="unable-to-get-workspace",
                 title="Couldn't get the workspace.",
@@ -210,10 +213,10 @@ class LookerClient:
                     "Please try again."
                 ),
                 response=response,
-            )
+            ) from error
         return response.json()["workspace_id"]
 
-    def update_workspace(self, workspace: str) -> None:
+    async def update_workspace(self, workspace: str) -> None:
         """Updates the session workspace.
 
         Args:
@@ -222,10 +225,10 @@ class LookerClient:
         logger.debug(f"Updating session to use the {workspace} workspace")
         url = utils.compose_url(self.api_url, path=["session"])
         body = {"workspace_id": workspace}
-        response = self.patch(url=url, json=body, timeout=TIMEOUT_SEC)
+        response = await self.patch(url=url, json=body, timeout=TIMEOUT_SEC)
         try:
             response.raise_for_status()
-        except requests.exceptions.HTTPError:
+        except httpx.HTTPStatusError as error:
             raise LookerApiError(
                 name="unable-to-update-workspace",
                 title="Couldn't update the session's workspace.",
@@ -237,10 +240,10 @@ class LookerClient:
                     "Spectacles is using, please save it and try again."
                 ),
                 response=response,
-            )
+            ) from error
         self.workspace = workspace
 
-    def get_all_branches(self, project: str) -> List[str]:
+    async def get_all_branches(self, project: str) -> List[str]:
         """Returns a list of git branches in the project repository.
 
         Args:
@@ -250,10 +253,10 @@ class LookerClient:
         url = utils.compose_url(
             self.api_url, path=["projects", project, "git_branches"]
         )
-        response = self.get(url=url, timeout=TIMEOUT_SEC)
+        response = await self.get(url=url, timeout=TIMEOUT_SEC)
         try:
             response.raise_for_status()
-        except requests.exceptions.HTTPError:
+        except httpx.HTTPStatusError as error:
             raise LookerApiError(
                 name="unable-to-get-branches",
                 title="Couldn't get all Git branches.",
@@ -263,11 +266,11 @@ class LookerClient:
                     "Please try again."
                 ),
                 response=response,
-            )
+            ) from error
 
         return [branch["name"] for branch in response.json()]
 
-    def checkout_branch(self, project: str, branch: str) -> None:
+    async def checkout_branch(self, project: str, branch: str) -> None:
         """Checks out a new git branch. Only works in dev workspace.
 
         Args:
@@ -277,10 +280,10 @@ class LookerClient:
         logger.debug(f"Setting project '{project}' branch to '{branch}'")
         url = utils.compose_url(self.api_url, path=["projects", project, "git_branch"])
         body = {"name": branch}
-        response = self.put(url=url, json=body, timeout=TIMEOUT_SEC)
+        response = await self.put(url=url, json=body, timeout=TIMEOUT_SEC)
         try:
             response.raise_for_status()
-        except requests.exceptions.HTTPError:
+        except httpx.HTTPStatusError as error:
             raise LookerApiError(
                 name="unable-to-checkout-branch",
                 title="Couldn't checkout Git branch.",
@@ -291,9 +294,9 @@ class LookerClient:
                     "please commit or revert them, then try again."
                 ),
                 response=response,
-            )
+            ) from error
 
-    def reset_to_remote(self, project: str) -> None:
+    async def reset_to_remote(self, project: str) -> None:
         """Reset a project development branch to the revision of the project that is on the remote.
 
         Args:
@@ -304,10 +307,10 @@ class LookerClient:
         url = utils.compose_url(
             self.api_url, path=["projects", project, "reset_to_remote"]
         )
-        response = self.post(url=url, timeout=TIMEOUT_SEC)
+        response = await self.post(url=url, timeout=TIMEOUT_SEC)
         try:
             response.raise_for_status()
-        except requests.exceptions.HTTPError:
+        except httpx.HTTPStatusError as error:
             raise LookerApiError(
                 name="unable-to-reset-remote",
                 title="Couldn't checkout Git branch.",
@@ -317,9 +320,9 @@ class LookerClient:
                     "to match remote. Please try again."
                 ),
                 response=response,
-            )
+            ) from error
 
-    def get_manifest(self, project: str) -> JsonDict:
+    async def get_manifest(self, project: str) -> JsonDict:
         """Gets all the dependent LookML projects defined in the manifest file.
 
         Args:
@@ -330,11 +333,11 @@ class LookerClient:
         """
         logger.debug("Getting manifest details")
         url = utils.compose_url(self.api_url, path=["projects", project, "manifest"])
-        response = self.get(url=url, timeout=TIMEOUT_SEC)
+        response = await self.get(url=url, timeout=TIMEOUT_SEC)
 
         try:
             response.raise_for_status()
-        except requests.exceptions.HTTPError:
+        except httpx.HTTPStatusError as error:
             raise LookerApiError(
                 name="unable-to-get-manifest",
                 title="Couldn't retrieve project manifest.",
@@ -345,13 +348,13 @@ class LookerClient:
                     "then try again."
                 ),
                 response=response,
-            )
+            ) from error
 
         manifest = response.json()
 
         return manifest
 
-    def get_active_branch(self, project: str) -> JsonDict:
+    async def get_active_branch(self, project: str) -> JsonDict:
         """Gets the active branch for the user in the given project.
 
         Args:
@@ -362,11 +365,11 @@ class LookerClient:
         """
         logger.debug(f"Getting active branch for project '{project}'")
         url = utils.compose_url(self.api_url, path=["projects", project, "git_branch"])
-        response = self.get(url=url, timeout=TIMEOUT_SEC)
+        response = await self.get(url=url, timeout=TIMEOUT_SEC)
 
         try:
             response.raise_for_status()
-        except requests.exceptions.HTTPError:
+        except httpx.HTTPStatusError as error:
             raise LookerApiError(
                 name="unable-to-get-active-branch",
                 title="Couldn't determine active Git branch.",
@@ -377,19 +380,21 @@ class LookerClient:
                     "has the correct permissions and try again."
                 ),
                 response=response,
-            )
+            ) from error
 
         branch_name = response.json()["name"]
         logger.debug(f"The active branch is '{branch_name}'")
 
         return response.json()
 
-    def get_active_branch_name(self, project: str) -> str:
+    async def get_active_branch_name(self, project: str) -> str:
         """Helper method to return only the branch name."""
-        full_response = self.get_active_branch(project)
+        full_response = await self.get_active_branch(project)
         return full_response["name"]
 
-    def create_branch(self, project: str, branch: str, ref: Optional[str] = None):
+    async def create_branch(
+        self, project: str, branch: str, ref: Optional[str] = None
+    ) -> None:
         """Creates a branch in the given project.
 
         Args:
@@ -417,20 +422,20 @@ class LookerClient:
         logger.debug(message)
 
         url = utils.compose_url(self.api_url, path=["projects", project, "git_branch"])
-        response = self.post(url=url, json=body, timeout=TIMEOUT_SEC)
+        response = await self.post(url=url, json=body, timeout=TIMEOUT_SEC)
 
         try:
             response.raise_for_status()
-        except requests.exceptions.HTTPError:
+        except httpx.HTTPStatusError as error:
             raise LookerApiError(
                 name="unable-to-create-branch",
                 title="Couldn't create new Git branch.",
                 status=response.status_code,
                 detail=detail,
                 response=response,
-            )
+            ) from error
 
-    def hard_reset_branch(self, project: str, branch: str, ref: str):
+    async def hard_reset_branch(self, project: str, branch: str, ref: str) -> None:
         """Hard resets a branch to the ref prodvided.
 
         DANGER: hard reset will be force pushed to the remote. Unsaved changes and
@@ -447,11 +452,11 @@ class LookerClient:
 
         body = {"name": branch, "ref": ref}
         url = utils.compose_url(self.api_url, path=["projects", project, "git_branch"])
-        response = self.put(url=url, json=body, timeout=TIMEOUT_SEC)
+        response = await self.put(url=url, json=body, timeout=TIMEOUT_SEC)
 
         try:
             response.raise_for_status()
-        except requests.exceptions.HTTPError:
+        except httpx.HTTPStatusError as error:
             raise LookerApiError(
                 name="unable-to-update-branch",
                 title="Couldn't update Git branch.",
@@ -462,9 +467,9 @@ class LookerClient:
                     "Please try again."
                 ),
                 response=response,
-            )
+            ) from error
 
-    def delete_branch(self, project: str, branch: str):
+    async def delete_branch(self, project: str, branch: str):
         """Deletes a branch in the given project.
 
         Args:
@@ -476,11 +481,11 @@ class LookerClient:
         url = utils.compose_url(
             self.api_url, path=["projects", project, "git_branch", branch]
         )
-        response = self.delete(url=url, timeout=TIMEOUT_SEC)
+        response = await self.delete(url=url, timeout=TIMEOUT_SEC)
 
         try:
             response.raise_for_status()
-        except requests.exceptions.HTTPError:
+        except httpx.HTTPStatusError as error:
             raise LookerApiError(
                 name="unable-to-delete-branch",
                 title="Couldn't delete Git branch.",
@@ -490,9 +495,9 @@ class LookerClient:
                     f"in project '{project}'. Please try again."
                 ),
                 response=response,
-            )
+            ) from error
 
-    def all_lookml_tests(self, project: str) -> List[JsonDict]:
+    async def all_lookml_tests(self, project: str) -> List[JsonDict]:
         """Gets all LookML/data tests for a given project.
 
         Args:
@@ -506,11 +511,11 @@ class LookerClient:
         url = utils.compose_url(
             self.api_url, path=["projects", project, "lookml_tests"]
         )
-        response = self.get(url=url, timeout=TIMEOUT_SEC)
+        response = await self.get(url=url, timeout=TIMEOUT_SEC)
 
         try:
             response.raise_for_status()
-        except requests.exceptions.HTTPError:
+        except httpx.HTTPStatusError as error:
             raise LookerApiError(
                 name="unable-to-get-data-tests",
                 title="Couldn't retrieve all data tests.",
@@ -520,12 +525,12 @@ class LookerClient:
                     f"project '{project}'. Please try again."
                 ),
                 response=response,
-            )
+            ) from error
 
         return response.json()
 
-    def run_lookml_test(
-        self, project: str, model: str = None, test: str = None
+    async def run_lookml_test(
+        self, project: str, model: Optional[str] = None, test: Optional[str] = None
     ) -> List[JsonDict]:
         """Runs all LookML/data tests for a given project and model (optional)
 
@@ -558,11 +563,15 @@ class LookerClient:
             params["model"] = model
         if test is not None:
             params["test"] = test
-        response = self.get(url=url, params=params, timeout=TIMEOUT_SEC)
+        response = await self.get(
+            url=url,
+            params=params,
+            timeout=1800,  # 30m timeout for long-running tests
+        )
 
         try:
             response.raise_for_status()
-        except requests.exceptions.HTTPError:
+        except httpx.HTTPStatusError as error:
             raise LookerApiError(
                 name="unable-to-run-data-test",
                 title="Couldn't run data test.",
@@ -572,11 +581,11 @@ class LookerClient:
                     f"project '{project}'. Please try again."
                 ),
                 response=response,
-            )
+            ) from error
 
         return response.json()
 
-    def get_lookml_models(self, fields: Optional[List] = None) -> List[JsonDict]:
+    async def get_lookml_models(self, fields: Optional[List] = None) -> List[JsonDict]:
         """Gets all models and explores from the LookmlModel endpoint.
 
         Returns:
@@ -592,21 +601,21 @@ class LookerClient:
             params["fields"] = fields
 
         url = utils.compose_url(self.api_url, path=["lookml_models"], params=params)
-        response = self.get(url=url, timeout=TIMEOUT_SEC)
+        response = await self.get(url=url, timeout=TIMEOUT_SEC)
         try:
             response.raise_for_status()
-        except requests.exceptions.HTTPError:
+        except httpx.HTTPStatusError as error:
             raise LookerApiError(
                 name="unable-to-get-lookml",
                 title="Couldn't retrieve models and explores.",
                 status=response.status_code,
                 detail="Unable to retrieve LookML details. Please try again.",
                 response=response,
-            )
+            ) from error
 
         return response.json()
 
-    def get_lookml_dimensions(self, model: str, explore: str) -> List[str]:
+    async def get_lookml_dimensions(self, model: str, explore: str) -> List[str]:
         """Gets all dimensions for an explore from the LookmlModel endpoint.
 
         Args:
@@ -625,10 +634,10 @@ class LookerClient:
             path=["lookml_models", model, "explores", explore],
             params=params,
         )
-        response = self.get(url=url, timeout=TIMEOUT_SEC)
+        response = await self.get(url=url, timeout=TIMEOUT_SEC)
         try:
             response.raise_for_status()
-        except requests.exceptions.HTTPError:
+        except httpx.HTTPStatusError as error:
             raise LookerApiError(
                 name="unable-to-get-dimension-lookml",
                 title="Couldn't retrieve dimensions.",
@@ -638,12 +647,17 @@ class LookerClient:
                     f"for explore '{model}/{explore}'. Please try again."
                 ),
                 response=response,
-            )
+            ) from error
 
         return response.json()["fields"]["dimensions"]
 
-    def create_query(
-        self, model: str, explore: str, dimensions: List[str], fields: List = None
+    @backoff.on_exception(backoff.expo, BACKOFF_EXCEPTIONS, max_tries=5)
+    async def create_query(
+        self,
+        model: str,
+        explore: str,
+        dimensions: List[str],
+        fields: Optional[List] = None,
     ) -> Dict:
         """Creates a Looker async query for one or more specified dimensions.
 
@@ -676,10 +690,10 @@ class LookerClient:
             params["fields"] = fields
 
         url = utils.compose_url(self.api_url, path=["queries"], params=params)
-        response = self.post(url=url, json=body, timeout=TIMEOUT_SEC)
+        response = await self.post(url=url, json=body, timeout=TIMEOUT_SEC)
         try:
             response.raise_for_status()
-        except requests.exceptions.HTTPError:
+        except httpx.HTTPStatusError as error:
             raise LookerApiError(
                 name="unable-to-create-query",
                 title="Couldn't create query.",
@@ -690,12 +704,12 @@ class LookerClient:
                     "Please try again."
                 ),
                 response=response,
-            )
+            ) from error
 
         result = response.json()
-        query_id = result["id"]
+        query_id: str = result["id"]
         logger.debug(
-            "Query for %s/%s/%s created as query %d",
+            "Query for %s/%s/%s created as query %s",
             model,
             explore,
             "*" if len(dimensions) != 1 else dimensions[0],
@@ -703,7 +717,7 @@ class LookerClient:
         )
         return result
 
-    def create_query_task(self, query_id: int) -> str:
+    async def create_query_task(self, query_id: str) -> str:
         """Runs a previously created query asynchronously and returns the query task ID.
 
         If a ClientError or TimeoutError is received, attempts to retry.
@@ -718,18 +732,18 @@ class LookerClient:
 
         """
         # Using old-style string formatting so that strings are formatted lazily
-        logger.debug("Starting query %d", query_id)
+        logger.debug("Starting query %s", query_id)
         body = {"query_id": query_id, "result_format": "json_detail"}
         params = {"fields": ["id"]}
         url = utils.compose_url(self.api_url, path=["query_tasks"], params=params)
 
-        response = self.post(
+        response = await self.post(
             url=url, json=body, params={"cache": "false"}, timeout=TIMEOUT_SEC
         )
 
         try:
             response.raise_for_status()
-        except requests.exceptions.HTTPError:
+        except httpx.HTTPStatusError as error:
             raise LookerApiError(
                 name="unable-to-launch-query",
                 title="Couldn't launch query.",
@@ -739,14 +753,16 @@ class LookerClient:
                     f"query '{query_id}'. Please try again."
                 ),
                 response=response,
-            )
+            ) from error
 
         result = response.json()
         query_task_id = result["id"]
-        logger.debug("Query %d is running under query task %s", query_id, query_task_id)
+        logger.debug("Query %s is running under query task %s", query_id, query_task_id)
         return query_task_id
 
-    def get_query_task_multi_results(self, query_task_ids: List[str]) -> JsonDict:
+    async def get_query_task_multi_results(
+        self, query_task_ids: Tuple[str, ...]
+    ) -> JsonDict:
         """Returns query task results.
 
         If a ClientError or TimeoutError is received, attempts to retry.
@@ -763,7 +779,7 @@ class LookerClient:
             "Attempting to get results for %d query tasks", len(query_task_ids)
         )
         url = utils.compose_url(self.api_url, path=["query_tasks", "multi_results"])
-        response = self.get(
+        response = await self.get(
             url=url,
             params={"query_task_ids": ",".join(query_task_ids)},
             timeout=TIMEOUT_SEC,
@@ -771,7 +787,7 @@ class LookerClient:
 
         try:
             response.raise_for_status()
-        except requests.exceptions.HTTPError:
+        except httpx.HTTPStatusError as error:
             raise LookerApiError(
                 name="unable-to-get-query-results",
                 title="Couldn't get results for the specified query tasks.",
@@ -782,12 +798,12 @@ class LookerClient:
                     "Please try again."
                 ),
                 response=response,
-            )
+            ) from error
 
         result = response.json()
         return result
 
-    def cancel_query_task(self, query_task_id: str):
+    async def cancel_query_task(self, query_task_id: str) -> None:
         """Cancels a query task.
 
         Args:
@@ -796,59 +812,59 @@ class LookerClient:
         """
         logger.debug(f"Cancelling query task: {query_task_id}")
         url = utils.compose_url(self.api_url, path=["running_queries", query_task_id])
-        self.delete(url=url, timeout=TIMEOUT_SEC)
+        await self.delete(url=url, timeout=TIMEOUT_SEC)
 
         # No raise_for_status() here because Looker API seems to give a 404
         # if you try to cancel a finished query which can happen as part of cleanup
 
-    def content_validation(self) -> JsonDict:
+    async def content_validation(self) -> JsonDict:
         logger.debug("Validating all content in Looker")
         url = utils.compose_url(self.api_url, path=["content_validation"])
-        response = self.get(
+        response = await self.get(
             url=url, timeout=3600
         )  # 1 hour timeout for content validation
 
         try:
             response.raise_for_status()
-        except requests.exceptions.HTTPError:
+        except httpx.HTTPStatusError as error:
             raise LookerApiError(
                 name="unable-to-validate-content",
                 title="Couldn't validate Looks and Dashboards.",
                 status=response.status_code,
                 detail=("Failed to run the content validator. Please try again."),
                 response=response,
-            )
+            ) from error
 
         result = response.json()
         return result
 
-    def lookml_validation(self, project) -> JsonDict:
+    async def lookml_validation(self, project) -> JsonDict:
         logger.debug(f"Validating LookML for project '{project}'")
         url = utils.compose_url(self.api_url, path=["projects", project, "validate"])
-        response = self.post(url=url, timeout=TIMEOUT_SEC)
+        response = await self.post(url=url, timeout=TIMEOUT_SEC)
 
         try:
             response.raise_for_status()
-        except requests.exceptions.HTTPError:
+        except httpx.HTTPStatusError as error:
             raise LookerApiError(
                 name="unable-to-validate-lookml",
                 title=f"Couldn't validate LookML in project {project}.",
                 status=response.status_code,
                 detail=("Failed to run the LookML validator. Please try again."),
                 response=response,
-            )
+            ) from error
 
         result = response.json()
         return result
 
-    def cached_lookml_validation(self, project) -> Optional[JsonDict]:
+    async def cached_lookml_validation(self, project) -> Optional[JsonDict]:
         logger.debug(f"Getting cached LookML validation results for '{project}'")
         url = utils.compose_url(self.api_url, path=["projects", project, "validate"])
-        response = self.get(url=url, timeout=TIMEOUT_SEC)
+        response = await self.get(url=url, timeout=TIMEOUT_SEC)
 
         try:
             response.raise_for_status()
-        except requests.exceptions.HTTPError:
+        except httpx.HTTPStatusError as error:
             raise LookerApiError(
                 name="unable-to-get-cached-lookml-validation",
                 title=f"Couldn't get cached LookML validation results in project '{project}'.",
@@ -857,7 +873,7 @@ class LookerClient:
                     "Failed to get cached LookML valiation results. Please try again."
                 ),
                 response=response,
-            )
+            ) from error
 
         # If no cached validation results are available, Looker returns a 204 No Content.
         # The response has no payload. We should return None in this case and handle accordingly.
@@ -867,27 +883,27 @@ class LookerClient:
         result = response.json()
         return result
 
-    def all_folders(self) -> List[JsonDict]:
+    async def all_folders(self) -> List[JsonDict]:
         logger.debug("Getting information about all folders")
         url = utils.compose_url(self.api_url, path=["folders"])
-        response = self.get(url=url, timeout=TIMEOUT_SEC)
+        response = await self.get(url=url, timeout=TIMEOUT_SEC)
 
         try:
             response.raise_for_status()
-        except requests.exceptions.HTTPError:
+        except httpx.HTTPStatusError as error:
             raise LookerApiError(
                 name="unable-to-get-folders",
                 title="Couldn't obtain project folders.",
                 status=response.status_code,
                 detail=("Failed to get all folders."),
                 response=response,
-            )
+            ) from error
 
         result = response.json()
         return result
 
-    @backoff.on_exception(backoff.expo, (Timeout,), max_tries=2)
-    def run_query(self, query_id: int) -> str:
+    @backoff.on_exception(backoff.expo, BACKOFF_EXCEPTIONS, max_tries=5)
+    async def run_query(self, query_id: str) -> str:
         """Returns the compiled SQL for a given query ID.
 
         The corresponding Looker API endpoint allows us to run queries with a variety
@@ -901,10 +917,10 @@ class LookerClient:
         logger.debug("Retrieving the SQL for query ID %s", query_id)
 
         url = utils.compose_url(self.api_url, path=["queries", query_id, "run", "sql"])
-        response = self.get(url=url, timeout=TIMEOUT_SEC)
+        response = await self.get(url=url, timeout=TIMEOUT_SEC)
         try:
             response.raise_for_status()
-        except requests.exceptions.HTTPError as e:
+        except httpx.HTTPStatusError as e:
             if e.response.status_code == 404:
                 return (
                     "-- SQL could not be generated because of errors with this query."
@@ -919,17 +935,9 @@ class LookerClient:
                         "Please try again."
                     ),
                     response=response,
-                )
+                ) from e
 
         result = response.text
+        logger.debug("Retrieved compiled SQL for query %s", query_id)
 
         return result
-
-
-class NullAuth(requests.auth.AuthBase):
-    """A custom auth class which ensures requests does not override authorization
-    headers with netrc file credentials if present.
-    """
-
-    def __call__(self, r):
-        return r
